import numpy as np
import numpy.typing as npt

from ._types import PMMCoreJTaggedImage


def fix_tagged_shape(
    tagged_image: PMMCoreJTaggedImage,
) -> npt.NDArray[np.uint16]:
    """Convert a 1D tagged image into a 2D numpy image."""
    return np.reshape(
        tagged_image.pix,
        newshape=(tagged_image.tags["Height"], tagged_image.tags["Width"]),
    )



"""
This module is an example of a barebones writer plugin for napari.

It implements the Writer specification.
see: https://napari.org/plugins/guides.html?#writers

Replace code below according to your needs.
"""
from __future__ import annotations

from typing import (
    TYPE_CHECKING,
    Any,
    List,
    Protocol,
    Sequence,
    Tuple,
    TypedDict,
    Union,
)

import numpy as np
import numpy.typing as npt
from pycromanager import Core

__all__ = [
    "get_image",
]


class PTaggedImageDict(TypedDict):
    Height: int
    Width: int


class PTaggedImage(Protocol):
    pix: npt.NDArray
    tags: PTaggedImageDict


if TYPE_CHECKING:
    DataType = Union[Any, Sequence[Any]]
    FullLayerData = Tuple[DataType, dict, str]


def write_single_image(path: str, data: Any, meta: dict):
    """Writes a single image layer"""


def write_multiple(path: str, data: List[FullLayerData]):
    """Writes multiple layers of different types."""


def get_image() -> npt.NDArray:

    core = Core()  # type: ignore
    core.set_property("Core", "AutoShutter", 0)  # type: ignore
    core.snap_image()  # type: ignore
    tagged_image: PTaggedImage = core.get_tagged_image()  # type: ignore

    # pixels by default come out as a 1D array.
    # We can reshape them into an image
    pixels = np.reshape(
        tagged_image.pix,
        newshape=(
            tagged_image.tags["Height"],
            tagged_image.tags["Width"],
        ),
    )

    return pixels

    # if data is None:
    #     try:
    #         data = self._mmc.getLastImage()
    #     except (RuntimeError, IndexError):
    #         # circular buffer empty
    #         return



from typing import TypedDict

import numpy.typing as npt

__all__ = ["IMDAParams", "MDAPresets", "get_mda_presets"]


class IMDAParams(TypedDict, total=False):
    """Inputs for the multi-dimensional acquisition events."""

    num_time_points: int
    time_interval_s: float
    z_start: float
    z_end: float
    z_step: float
    channel_group: str
    channels: list
    channel_exposures_ms: list
    xy_positions: npt.ArrayLike
    xyz_positions: npt.ArrayLike
    order: str
    keep_shutter_open_between_channels: bool
    keep_shutter_open_between_z_steps: bool


class MDAPresets(TypedDict):
    Default: IMDAParams
    Simple: IMDAParams
    Detailed: IMDAParams


def get_mda_presets() -> MDAPresets:
    return MDAPresets(
        Default=IMDAParams(
            num_time_points=5,
            z_start=0,
            z_end=6,
            z_step=0.4,
        ),
        Simple=IMDAParams(
            num_time_points=2,
            z_start=0,
            z_end=2,
            z_step=0.1,
        ),
        Detailed=IMDAParams(
            num_time_points=10,
            z_start=0,
            z_end=12,
            z_step=0.2,
        ),
    )



# from PyQt5 import QtCore, QtGui, QtWidgets
from qtpy import QtCore, QtGui, QtWidgets


class RightPanel:
    def setupUi(self, Form: QtWidgets.QWidget) -> None:
        Form.setObjectName("Form")
        Form.resize(585, 800)
        self.layoutWidget = QtWidgets.QWidget(Form)
        self.layoutWidget.setGeometry(QtCore.QRect(150, 0, 430, 794))
        self.layoutWidget.setObjectName("layoutWidget")
        self.right_vertical_layout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.right_vertical_layout.setContentsMargins(0, 0, 0, 0)
        self.right_vertical_layout.setObjectName("right_vertical_layout")
        self.groupBox = QtWidgets.QGroupBox(self.layoutWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.groupBox.setFont(font)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.active_directory_line_edit = QtWidgets.QLineEdit(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.active_directory_line_edit.setFont(font)
        self.active_directory_line_edit.setAlignment(QtCore.Qt.AlignCenter)
        self.active_directory_line_edit.setReadOnly(True)
        self.active_directory_line_edit.setObjectName(
            "active_directory_line_edit"
        )
        self.horizontalLayout_11.addWidget(self.active_directory_line_edit)
        self.directory_browse_button = QtWidgets.QPushButton(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.directory_browse_button.setFont(font)
        self.directory_browse_button.setObjectName("directory_browse_button")
        self.horizontalLayout_11.addWidget(self.directory_browse_button)
        self.verticalLayout_3.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.active_file_line_edit = QtWidgets.QLineEdit(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.active_file_line_edit.setFont(font)
        self.active_file_line_edit.setAlignment(QtCore.Qt.AlignCenter)
        self.active_file_line_edit.setReadOnly(True)
        self.active_file_line_edit.setObjectName("active_file_line_edit")
        self.horizontalLayout_8.addWidget(self.active_file_line_edit)
        self.filename_browse_button = QtWidgets.QPushButton(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.filename_browse_button.setFont(font)
        self.filename_browse_button.setObjectName("filename_browse_button")
        self.horizontalLayout_8.addWidget(self.filename_browse_button)
        self.verticalLayout_3.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.comboBox = QtWidgets.QComboBox(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.comboBox.setFont(font)
        self.comboBox.setAutoFillBackground(False)
        self.comboBox.setEditable(False)
        self.comboBox.setCurrentText("")
        self.comboBox.setMaxVisibleItems(100)
        self.comboBox.setFrame(True)
        self.comboBox.setObjectName("comboBox")
        self.horizontalLayout_7.addWidget(self.comboBox)
        self.verticalLayout_3.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.load_button = QtWidgets.QPushButton(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.load_button.setFont(font)
        self.load_button.setObjectName("load_button")
        self.horizontalLayout_6.addWidget(self.load_button)
        self.scan_push_button = QtWidgets.QPushButton(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.scan_push_button.setFont(font)
        self.scan_push_button.setObjectName("scan_push_button")
        self.horizontalLayout_6.addWidget(self.scan_push_button)
        self.pushButton_2 = QtWidgets.QPushButton(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.pushButton_2.setFont(font)
        self.pushButton_2.setObjectName("pushButton_2")
        self.horizontalLayout_6.addWidget(self.pushButton_2)
        self.verticalLayout_3.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_9.addWidget(self.label_2)
        self.progressBar = QtWidgets.QProgressBar(self.groupBox)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setObjectName("progressBar")
        self.horizontalLayout_9.addWidget(self.progressBar)
        self.verticalLayout_3.addLayout(self.horizontalLayout_9)
        self.right_vertical_layout.addWidget(self.groupBox)
        self.groupBox_3 = QtWidgets.QGroupBox(self.layoutWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.groupBox_3.setFont(font)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_4.setContentsMargins(9, 20, -1, -1)
        self.verticalLayout_4.setSpacing(6)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.live_view_push_button = QtWidgets.QPushButton(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.live_view_push_button.setFont(font)
        self.live_view_push_button.setObjectName("live_view_push_button")
        self.verticalLayout_5.addWidget(self.live_view_push_button)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.exposure_spin_box = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.exposure_spin_box.setFont(font)
        self.exposure_spin_box.setObjectName("doubleSpinBox")
        self.horizontalLayout_13.addWidget(self.exposure_spin_box)
        self.label_5 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_13.addWidget(self.label_5)
        self.verticalLayout_5.addLayout(self.horizontalLayout_13)
        self.pushButton_19 = QtWidgets.QPushButton(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.pushButton_19.setFont(font)
        self.pushButton_19.setObjectName("pushButton_19")
        self.verticalLayout_5.addWidget(self.pushButton_19)
        self.horizontalLayout_12.addLayout(self.verticalLayout_5)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.checkBox = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.checkBox.setFont(font)
        self.checkBox.setObjectName("checkBox")
        self.verticalLayout_7.addWidget(self.checkBox)
        self.checkBox_2 = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.checkBox_2.setFont(font)
        self.checkBox_2.setObjectName("checkBox_2")
        self.verticalLayout_7.addWidget(self.checkBox_2)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.checkBox_3 = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.checkBox_3.setFont(font)
        self.checkBox_3.setObjectName("checkBox_3")
        self.gridLayout_3.addWidget(self.checkBox_3, 1, 1, 1, 1)
        self.checkBox_6 = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.checkBox_6.setFont(font)
        self.checkBox_6.setObjectName("checkBox_6")
        self.gridLayout_3.addWidget(self.checkBox_6, 1, 2, 1, 1)
        self.checkBox_4 = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.checkBox_4.setFont(font)
        self.checkBox_4.setObjectName("checkBox_4")
        self.gridLayout_3.addWidget(self.checkBox_4, 0, 1, 1, 1)
        self.checkBox_5 = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.checkBox_5.setFont(font)
        self.checkBox_5.setObjectName("checkBox_5")
        self.gridLayout_3.addWidget(self.checkBox_5, 0, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(
            40,
            20,
            QtWidgets.QSizePolicy.Expanding,
            QtWidgets.QSizePolicy.Minimum,
        )
        self.gridLayout_3.addItem(spacerItem, 0, 0, 1, 1)
        self.verticalLayout_7.addLayout(self.gridLayout_3)
        self.horizontalLayout_12.addLayout(self.verticalLayout_7)
        self.verticalLayout_4.addLayout(self.horizontalLayout_12)
        self.groupBox_4 = QtWidgets.QGroupBox(self.groupBox_3)
        self.groupBox_4.setEnabled(True)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.groupBox_4.setFont(font)
        self.groupBox_4.setAutoFillBackground(False)
        self.groupBox_4.setFlat(False)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.pushButton_13 = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.pushButton_13.sizePolicy().hasHeightForWidth()
        )
        self.pushButton_13.setSizePolicy(sizePolicy)
        self.pushButton_13.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(
            QtGui.QPixmap("icons/fast-forward-left.png"),
            QtGui.QIcon.Normal,
            QtGui.QIcon.On,
        )
        self.pushButton_13.setIcon(icon)
        self.pushButton_13.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_13.setObjectName("pushButton_13")
        self.gridLayout_2.addWidget(self.pushButton_13, 3, 0, 1, 1)
        self.pushButton_15 = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButton_15.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(
            QtGui.QPixmap("icons/fast-forward-down.png"),
            QtGui.QIcon.Normal,
            QtGui.QIcon.On,
        )
        self.pushButton_15.setIcon(icon1)
        self.pushButton_15.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_15.setObjectName("pushButton_15")
        self.gridLayout_2.addWidget(self.pushButton_15, 5, 6, 1, 1)
        self.pushButton_7 = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.pushButton_7.sizePolicy().hasHeightForWidth()
        )
        self.pushButton_7.setSizePolicy(sizePolicy)
        self.pushButton_7.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(
            QtGui.QPixmap("icons/play-down.png"),
            QtGui.QIcon.Normal,
            QtGui.QIcon.On,
        )
        self.pushButton_7.setIcon(icon2)
        self.pushButton_7.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_7.setObjectName("pushButton_7")
        self.gridLayout_2.addWidget(self.pushButton_7, 4, 2, 1, 1)
        self.pushButton_16 = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButton_16.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(
            QtGui.QPixmap("icons/play-up.png"),
            QtGui.QIcon.Normal,
            QtGui.QIcon.On,
        )
        self.pushButton_16.setIcon(icon3)
        self.pushButton_16.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_16.setObjectName("pushButton_16")
        self.gridLayout_2.addWidget(self.pushButton_16, 2, 6, 1, 1)
        self.pushButton_17 = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButton_17.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(
            QtGui.QPixmap("icons/fast-forward-up.png"),
            QtGui.QIcon.Normal,
            QtGui.QIcon.On,
        )
        self.pushButton_17.setIcon(icon4)
        self.pushButton_17.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_17.setObjectName("pushButton_17")
        self.gridLayout_2.addWidget(self.pushButton_17, 1, 6, 1, 1)
        self.pushButton_9 = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.pushButton_9.sizePolicy().hasHeightForWidth()
        )
        self.pushButton_9.setSizePolicy(sizePolicy)
        self.pushButton_9.setText("")
        self.pushButton_9.setIcon(icon4)
        self.pushButton_9.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_9.setFlat(False)
        self.pushButton_9.setObjectName("pushButton_9")
        self.gridLayout_2.addWidget(self.pushButton_9, 1, 2, 1, 1)
        self.pushButton_6 = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.pushButton_6.sizePolicy().hasHeightForWidth()
        )
        self.pushButton_6.setSizePolicy(sizePolicy)
        self.pushButton_6.setText("")
        self.pushButton_6.setIcon(icon1)
        self.pushButton_6.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_6.setObjectName("pushButton_6")
        self.gridLayout_2.addWidget(self.pushButton_6, 5, 2, 1, 1)
        self.pushButton_12 = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.pushButton_12.sizePolicy().hasHeightForWidth()
        )
        self.pushButton_12.setSizePolicy(sizePolicy)
        self.pushButton_12.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(
            QtGui.QPixmap("icons/play-left.png"),
            QtGui.QIcon.Normal,
            QtGui.QIcon.On,
        )
        self.pushButton_12.setIcon(icon5)
        self.pushButton_12.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_12.setObjectName("pushButton_12")
        self.gridLayout_2.addWidget(self.pushButton_12, 3, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.groupBox_4)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_4.setFont(font)
        self.label_4.setAlignment(QtCore.Qt.AlignCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 3, 6, 1, 1)
        self.pushButton_10 = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.pushButton_10.sizePolicy().hasHeightForWidth()
        )
        self.pushButton_10.setSizePolicy(sizePolicy)
        self.pushButton_10.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(
            QtGui.QPixmap("icons/play-right.png"),
            QtGui.QIcon.Normal,
            QtGui.QIcon.On,
        )
        self.pushButton_10.setIcon(icon6)
        self.pushButton_10.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_10.setObjectName("pushButton_10")
        self.gridLayout_2.addWidget(self.pushButton_10, 3, 3, 1, 1)
        self.pushButton_11 = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.pushButton_11.sizePolicy().hasHeightForWidth()
        )
        self.pushButton_11.setSizePolicy(sizePolicy)
        self.pushButton_11.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(
            QtGui.QPixmap("icons/fast-forward-right.png"),
            QtGui.QIcon.Normal,
            QtGui.QIcon.On,
        )
        self.pushButton_11.setIcon(icon7)
        self.pushButton_11.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_11.setObjectName("pushButton_11")
        self.gridLayout_2.addWidget(self.pushButton_11, 3, 4, 1, 1)
        self.fine_xy_up_button = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.fine_xy_up_button.sizePolicy().hasHeightForWidth()
        )
        self.fine_xy_up_button.setSizePolicy(sizePolicy)
        self.fine_xy_up_button.setText("")
        self.fine_xy_up_button.setIcon(icon3)
        self.fine_xy_up_button.setIconSize(QtCore.QSize(24, 24))
        self.fine_xy_up_button.setObjectName("fine_xy_up_button")
        self.gridLayout_2.addWidget(self.fine_xy_up_button, 2, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.groupBox_4)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_3.setFont(font)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 3, 2, 1, 1)
        self.pushButton_14 = QtWidgets.QPushButton(self.groupBox_4)
        self.pushButton_14.setText("")
        self.pushButton_14.setIcon(icon2)
        self.pushButton_14.setIconSize(QtCore.QSize(24, 24))
        self.pushButton_14.setObjectName("pushButton_14")
        self.gridLayout_2.addWidget(self.pushButton_14, 4, 6, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(
            40,
            20,
            QtWidgets.QSizePolicy.Expanding,
            QtWidgets.QSizePolicy.Minimum,
        )
        self.gridLayout_2.addItem(spacerItem1, 3, 5, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(
            40,
            20,
            QtWidgets.QSizePolicy.Expanding,
            QtWidgets.QSizePolicy.Minimum,
        )
        self.gridLayout_2.addItem(spacerItem2, 3, 7, 1, 1)
        self.verticalLayout_4.addWidget(self.groupBox_4)
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.xy_spin_box_fine = QtWidgets.QSpinBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.xy_spin_box_fine.setFont(font)
        self.xy_spin_box_fine.setMaximum(100000)
        self.xy_spin_box_fine.setObjectName("spinBox")
        self.horizontalLayout_5.addWidget(self.xy_spin_box_fine)
        self.label_7 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_5.addWidget(self.label_7)
        spacerItem3 = QtWidgets.QSpacerItem(
            40,
            20,
            QtWidgets.QSizePolicy.Expanding,
            QtWidgets.QSizePolicy.Minimum,
        )
        self.horizontalLayout_5.addItem(spacerItem3)
        self.z_fine_spinbox = QtWidgets.QSpinBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.z_fine_spinbox.setFont(font)
        self.z_fine_spinbox.setMaximum(100000)
        self.z_fine_spinbox.setObjectName("spinBox_2")
        self.horizontalLayout_5.addWidget(self.z_fine_spinbox)
        self.label_6 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_5.addWidget(self.label_6)
        self.verticalLayout_11.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.xy_spin_box_coarse = QtWidgets.QSpinBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.xy_spin_box_coarse.setFont(font)
        self.xy_spin_box_coarse.setMaximum(100000)
        self.xy_spin_box_coarse.setSingleStep(1000)
        self.xy_spin_box_coarse.setObjectName("spinBox_4")
        self.horizontalLayout_14.addWidget(self.xy_spin_box_coarse)
        self.label_9 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_14.addWidget(self.label_9)
        spacerItem4 = QtWidgets.QSpacerItem(
            40,
            20,
            QtWidgets.QSizePolicy.Expanding,
            QtWidgets.QSizePolicy.Minimum,
        )
        self.horizontalLayout_14.addItem(spacerItem4)
        self.spinBox_3 = QtWidgets.QSpinBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.spinBox_3.setFont(font)
        self.spinBox_3.setMaximum(100000)
        self.spinBox_3.setSingleStep(1000)
        self.spinBox_3.setObjectName("spinBox_3")
        self.horizontalLayout_14.addWidget(self.spinBox_3)
        self.label_8 = QtWidgets.QLabel(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_14.addWidget(self.label_8)
        self.verticalLayout_11.addLayout(self.horizontalLayout_14)
        self.verticalLayout_4.addLayout(self.verticalLayout_11)
        self.right_vertical_layout.addWidget(self.groupBox_3)
        self.groupBox_2 = QtWidgets.QGroupBox(Form)
        self.groupBox_2.setGeometry(QtCore.QRect(0, 0, 146, 776))
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.pushButton_20 = QtWidgets.QPushButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.pushButton_20.setFont(font)
        self.pushButton_20.setObjectName("pushButton_20")
        self.verticalLayout_8.addWidget(self.pushButton_20)
        spacerItem5 = QtWidgets.QSpacerItem(
            20,
            40,
            QtWidgets.QSizePolicy.Minimum,
            QtWidgets.QSizePolicy.Expanding,
        )
        self.verticalLayout_8.addItem(spacerItem5)
        self.constrastGroupBox = QtWidgets.QGroupBox(self.groupBox_2)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.constrastGroupBox.setFont(font)
        self.constrastGroupBox.setObjectName("constrastGroupBox")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.constrastGroupBox)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.onRadio = QtWidgets.QRadioButton(self.constrastGroupBox)
        self.onRadio.setChecked(True)
        self.onRadio.setObjectName("onRadio")
        self.verticalLayout_9.addWidget(self.onRadio)
        self.offRadio = QtWidgets.QRadioButton(self.constrastGroupBox)
        self.offRadio.setObjectName("offRadio")
        self.verticalLayout_9.addWidget(self.offRadio)
        self.verticalLayout_8.addWidget(self.constrastGroupBox)
        self.colormapGroupBox = QtWidgets.QGroupBox(self.groupBox_2)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.colormapGroupBox.setFont(font)
        self.colormapGroupBox.setObjectName("colormapGroupBox")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.colormapGroupBox)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.grayscale = QtWidgets.QRadioButton(self.colormapGroupBox)
        self.grayscale.setChecked(True)
        self.grayscale.setObjectName("grayscale")
        self.verticalLayout_10.addWidget(self.grayscale)
        self.heat = QtWidgets.QRadioButton(self.colormapGroupBox)
        self.heat.setObjectName("heat")
        self.verticalLayout_10.addWidget(self.heat)
        self.threshold = QtWidgets.QRadioButton(self.colormapGroupBox)
        self.threshold.setObjectName("threshold")
        self.verticalLayout_10.addWidget(self.threshold)
        self.verticalLayout_8.addWidget(self.colormapGroupBox)

        self.retranslateUi(Form)
        self.comboBox.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.groupBox.setTitle(_translate("Form", "Scanning"))
        self.active_directory_line_edit.setText(
            _translate("Form", "Directory")
        )
        self.directory_browse_button.setText(_translate("Form", "Browse"))
        self.active_file_line_edit.setText(_translate("Form", "Filename"))
        self.filename_browse_button.setText(_translate("Form", "Browse"))
        self.load_button.setText(_translate("Form", "Load"))
        self.scan_push_button.setText(_translate("Form", "Scan"))
        self.pushButton_2.setText(_translate("Form", "Abort"))
        self.label_2.setText(_translate("Form", "Scan Progress"))
        self.groupBox_3.setTitle(_translate("Form", "Instrument Control"))
        self.live_view_push_button.setText(_translate("Form", "Live View"))
        self.label_5.setText(_translate("Form", "Exposure (ms)"))
        self.pushButton_19.setText(_translate("Form", "Autofocus"))
        self.checkBox.setText(_translate("Form", "Ring Light"))
        self.checkBox_2.setText(_translate("Form", "Scan Light"))
        self.checkBox_3.setText(_translate("Form", "B"))
        self.checkBox_6.setText(_translate("Form", "D"))
        self.checkBox_4.setText(_translate("Form", "A"))
        self.checkBox_5.setText(_translate("Form", "C"))
        self.groupBox_4.setTitle(_translate("Form", "Move"))
        self.label_4.setText(_translate("Form", "Z"))
        self.label_3.setText(_translate("Form", "X-Y"))
        self.label_7.setText(_translate("Form", "Fine step"))
        self.label_6.setText(_translate("Form", "Fine step    "))
        self.label_9.setText(_translate("Form", "Course step"))
        self.label_8.setText(_translate("Form", "Course step"))
        self.groupBox_2.setTitle(_translate("Form", "GroupBox"))
        self.pushButton_20.setText(_translate("Form", "Go To Image"))
        self.constrastGroupBox.setTitle(_translate("Form", "Contrast"))
        self.onRadio.setText(_translate("Form", "On"))
        self.offRadio.setText(_translate("Form", "Off"))
        self.colormapGroupBox.setTitle(_translate("Form", "Colormap"))
        self.grayscale.setText(_translate("Form", "Grayscale"))
        self.heat.setText(_translate("Form", "Heat"))
        self.threshold.setText(_translate("Form", "Threshold"))



"""
This module is an example of a barebones QWidget plugin for napari

It implements the Widget specification.
see: https://napari.org/plugins/guides.html?#widgets

Replace code below according to your needs.
"""

from __future__ import annotations

import enum
from pathlib import Path
from typing import cast

import numpy as np
import numpy.typing as npt
from napari.viewer import Viewer
from pycromanager import Acquisition, Core, Dataset, multi_d_acquisition_events
from qtpy import QtWidgets as qtw
from qtpy.QtCore import QObject, QRunnable, QThreadPool, QTimer, Signal, Slot

from . import _live_view
from ._presets import IMDAParams, get_mda_presets
from ._right_panel import RightPanel
from ._types import PCore


class ExampleQWidget(qtw.QWidget, RightPanel):
    def __init__(self, napari_viewer: Viewer):
        super().__init__()
        self.setupUi(self)
        self.setMinimumWidth(600)
        self.core = cast(PCore, Core())  # type: ignore

        self.streaming_timer: QTimer | None = None

        self.comboBox.addItems(get_mda_presets().keys())

        self.comboBox.currentIndexChanged.connect(
            lambda: print(get_mda_presets()[self.comboBox.currentText()])
        )

        xy_spinbox_fine_default = 1
        self.xy_spin_box_fine.setValue(xy_spinbox_fine_default)

        self.z_fine_step = 1
        self.z_fine_spinbox.setValue(self.z_fine_step)

        self.xy_coarse_step = 3
        self.xy_spin_box_coarse.setValue(self.xy_coarse_step)

        def move_stage(
            direction: XYDirection | ZDirection,
            size: float,
        ) -> None:

            match direction:
                case XYDirection.UP:
                    self.core.set_relative_xy_position(0, size)
                case XYDirection.DOWN:
                    self.core.set_relative_xy_position(0, -size)
                case XYDirection.LEFT:
                    self.core.set_relative_xy_position(-size, 0)
                case XYDirection.RIGHT:
                    self.core.set_relative_xy_position(size, 0)
                case ZDirection.UP:
                    self.core.set_relative_position(size)
                case ZDirection.DOWN:
                    self.core.set_relative_position(-size)
                case _:
                    raise Exception("Not implemented")

            print(
                f"x:{self.core.get_x_position():.2f}\t"
                f"y:{self.core.get_y_position():.2f}\t"
                f"z:{self.core.get_position():.2f}"
            )

        def handle_load_click() -> None:
            self.core.home(self.core.get_xy_stage_device())
            print(
                f"x:{self.core.get_x_position():.2f}\t"
                f"y:{self.core.get_y_position():.2f}\t"
                f"z:{self.core.get_position():.2f}"
            )

        self.load_button.clicked.connect(handle_load_click)

        self.fine_xy_up_button.clicked.connect(
            lambda: move_stage(XYDirection.UP, self.xy_spin_box_fine.value())
        )

        self.pushButton_9.clicked.connect(
            lambda: move_stage(XYDirection.UP, self.xy_spin_box_coarse.value())
        )

        self.pushButton_12.clicked.connect(
            lambda: move_stage(XYDirection.LEFT, self.xy_spin_box_fine.value())
        )
        self.pushButton_13.clicked.connect(
            lambda: move_stage(
                XYDirection.LEFT, self.xy_spin_box_coarse.value()
            )
        )
        self.pushButton_7.clicked.connect(
            lambda: move_stage(XYDirection.DOWN, self.xy_spin_box_fine.value())
        )
        self.pushButton_6.clicked.connect(
            lambda: move_stage(
                XYDirection.DOWN, self.xy_spin_box_coarse.value()
            )
        )
        self.pushButton_10.clicked.connect(
            lambda: move_stage(
                XYDirection.RIGHT, self.xy_spin_box_fine.value()
            )
        )
        self.pushButton_11.clicked.connect(
            lambda: move_stage(
                XYDirection.RIGHT, self.xy_spin_box_coarse.value()
            )
        )
        self.pushButton_16.clicked.connect(
            lambda: move_stage(ZDirection.UP, self.z_fine_spinbox.value())
        )
        self.pushButton_14.clicked.connect(
            lambda: move_stage(ZDirection.DOWN, self.z_fine_spinbox.value())
        )

        self.z_coarse_step = 3
        self.spinBox_3.setValue(self.z_coarse_step)

        self.exposure_spin_box.setValue(self.core.get_exposure())

        def set_exposure(new_value: int) -> None:
            """Spinbox callback for `valueChanged` signal."""
            self.core.set_exposure(new_value)
            print("exposure set to", self.core.get_exposure())

        self.exposure_spin_box.valueChanged.connect(set_exposure)

        self.save_path = Path(__file__).parent / "images"
        self.active_directory_line_edit.setText(str(self.save_path))

        self.data = None
        self.viewer = napari_viewer

        self.threadpool = QThreadPool()

        self.scan_push_button.clicked.connect(self.execute_acquisition)

        def browse_directory() -> None:
            """Opens a file dialog to select a directory."""

            directory = qtw.QFileDialog.getExistingDirectory(
                self, "Select a directory", str(self.save_path.parent)
            )
            self.active_directory_line_edit.setText(directory)

        self.directory_browse_button.clicked.connect(browse_directory)

        self.live_view_push_button.setCheckable(True)
        self.live_view_push_button.clicked.connect(self.execute_live_view)

    def active_mda_params(self) -> IMDAParams:
        """Lookup the current MDA preset from the combobox."""
        return get_mda_presets()[self.comboBox.currentText()]

    def execute_acquisition(self) -> None:
        worker = Worker(self)
        worker.signals.data.connect(self.receive_data)
        worker.signals.progress.connect(self.update_progress)
        self.threadpool.start(worker)

    def execute_live_view(self, is_checked: bool) -> None:
        worker = _live_view.Worker(self)
        worker.signals.data.connect(self.receive_data)

        if is_checked:
            self.live_view_push_button.setStyleSheet("color: red;")
            self.live_view_push_button.setText("Stop live view")
            self.threadpool.start(worker)
        else:
            self.live_view_push_button.setStyleSheet("color: white;")
            self.live_view_push_button.setText("Start live view")

    def update_progress(self, progress: int) -> None:
        """Updates the progress bar."""

        # disable button while progress is active
        self.scan_push_button.setEnabled(False)
        self.progressBar.setValue(progress)
        if progress == 100:
            self.scan_push_button.setEnabled(True)
            self.progressBar.setValue(0)

    def receive_data(self, image: npt.NDArray) -> None:
        """Callback to receive the emitted image events from a pyqt
        `QRunnable` worker."""
        try:
            preview_layer = self.viewer.layers["preview"]  # type: ignore
            preview_layer.data = image
        except KeyError:
            preview_layer = self.viewer.add_image(image, name="preview")
            print("napari has", len(self.viewer.layers), "layers")


class WorkerSignals(QObject):
    """
    Defines the signals available from a running worker thread.

    :param data: A Signal for sending data from the worker to the main thread.
    :param progress: A Signal for sending progress updates to the main thread.
    """

    data = Signal(np.ndarray)
    progress = Signal(int)


class Worker(QRunnable):
    """
    Worker thread

    :param args: Arguments to make available to the run code
    :param kwargs: Keywords arguments to make available to the run
    :code
    :
    """

    def __init__(self, parent: ExampleQWidget):
        super().__init__()
        self.parent = parent

        self.signals = (
            WorkerSignals()
        )  # Create an instance of our signals class.
        self.event_count = 0
        self.event_total = 0

    @Slot()
    def run(self) -> None:
        """
        Initialize the runner function with passed self.args,
        self.kwargs.
        """

        def on_image_saved(axes: dict[str, int], dataset: Dataset) -> None:
            """Callback for when pycromanager saves an image.

            - Emits a data(image) event event signal.
            - Emits a progress(int) event signal.

            To be passed to the Acuisition `img_save_fn` callback.
            """
            self.event_count += 1
            self.signals.progress.emit(
                int(self.event_count / self.event_total * 100)
            )
            image = dataset.read_image(**axes)
            self.signals.data.emit(image)

        try:
            self.parent.core.stop_sequence_acquisition()
        except Exception as e:
            print(e)

        with Acquisition(
            directory=str(self.parent.save_path),
            name="tcz_acq",
            image_saved_fn=on_image_saved,
            show_display=False,
            debug=False,
        ) as acq:

            events = multi_d_acquisition_events(
                **self.parent.active_mda_params()
            )
            self.event_total = len(events)

            acq.acquire(events)


class XYDirection(enum.Enum):
    UP = enum.auto()
    DOWN = enum.auto()
    LEFT = enum.auto()
    RIGHT = enum.auto()


class ZDirection(enum.Enum):
    UP = enum.auto()
    DOWN = enum.auto()



from __future__ import annotations

from typing import TYPE_CHECKING

import numpy as np
from qtpy.QtCore import QObject, QRunnable, Signal, Slot

from ._utils import fix_tagged_shape

if TYPE_CHECKING:
    from ._widget import ExampleQWidget


class WorkerSignals(QObject):
    """
    Defines the signals available from a running worker thread.

    :param data: A Signal for sending data from the worker to the main thread.
    :param progress: A Signal for sending progress updates to the main thread.
    """

    data = Signal(np.ndarray)
    is_active = Signal(bool)


class Worker(QRunnable):
    """
    Worker thread

    :param args: Arguments to make available to the run code
    :param kwargs: Keywords arguments to make available to the run
    :code
    :
    """

    def __init__(self, parent: ExampleQWidget):
        super().__init__()
        self.parent = parent

        self.signals = (
            WorkerSignals()
        )  # Create an instance of our signals class.
        self.event_count = 0
        self.event_total = 0

    @Slot()
    def run(self) -> None:
        """
        Initialize the runner function with passed self.args,
        self.kwargs.
        """
        print("Starting live view")
        core = self.parent.core
        frame_rate = int(1000 / 24)

        try:
            self.parent.core.start_continuous_sequence_acquisition(frame_rate)
        except Exception as e:
            print(e)

        while self.parent.live_view_push_button.isChecked():
            try:
                image = fix_tagged_shape(core.get_last_tagged_image())
                self.signals.data.emit(image)
            except Exception as e:
                print(type(e))

        print("Stopping live view")



from __future__ import annotations

from typing import Literal, Protocol, TypedDict

import numpy as np
import numpy.typing as npt

__all__ = ["PCore", "TaggedImageTags", "PMMCoreJTaggedImage"]


class PCore(Protocol):
    """Derived from:
    https://valelab4.ucsf.edu/~MM/doc/MMCore/html/class_c_m_m_core.html
    """

    def set_xy_position(self, x: float, y: float) -> None:
        ...

    def set_relative_xy_position(self, x: float, y: float) -> None:
        ...

    def get_x_position(self) -> float:
        ...

    def get_y_position(self) -> float:
        ...

    def get_position(self) -> int | float:
        """Set z-axis position."""
        ...

    def set_relative_position(self, size: float) -> None:
        """Get the z-axis position."""
        ...

    def get_exposure(self) -> float | int:
        ...

    def set_exposure(self, value: float | int) -> None:
        ...

    def start_continuous_sequence_acquisition(self, ms_interval: int) -> None:
        ...

    def get_last_tagged_image(
        self,
    ) -> PMMCoreJTaggedImage:
        """Retrieve the last image when running in continuous
        acquisition mode."""
        ...

    def stop_sequence_acquisition(self) -> None:
        ...

    def home(self, stage_label: Literal["XY"] | str) -> None:
        ...

    def get_xy_stage_device(self) -> Literal["XY"] | str:
        ...


class TaggedImageTags(TypedDict):
    Height: int
    Width: int


class PMMCoreJTaggedImage(Protocol):
    pix: npt.NDArray[np.uint16]
    tags: TaggedImageTags



__version__ = "0.0.1"

from ._reader import napari_get_reader
from ._sample_data import make_sample_data
from ._widget import ExampleQWidget
from ._writer import write_multiple, write_single_image

__all__ = (
    "napari_get_reader",
    "write_single_image",
    "write_multiple",
    "make_sample_data",
    "ExampleQWidget",
)
